<?php

class Controller_Api_V0_StudyPlan extends Controller_Handler_Api
{
   private function normalizeDisciplinesData($data) {
        if ( !isset($data->id) && isset($data->externalID) ) {
            $ID = Model_Discipline::withExternalID($data->externalID);
            if ( $ID != -1 )
                $data->id = $ID;
        }

        foreach ($data->teachersHashSnils as $teacherHashSnils) {
            $ID = Model_Teacher::withINILA($teacherHashSnils);
            if ( $ID != -1 )
                $data->teacherIDs[] = $ID;
        }

        /*
         * Если нет ни одного преподавателя, то ругаемся в логи и пропускаем дисциплину до лучших времен
         */
        if ( !isset($data->teacherIDs) || count($data->teacherIDs) == 0 ) {
            Log::instance()->add(Log::WARNING, "A discipline without teachers is arrived: {0}",
                array(
                    '{0}'   => print_r($data, TRUE)
                )
            );
            return null;
        }

        switch ($data->type) {
            case 'Экзамен':
                $data->type = Model_Discipline::EXAM;
                break;
            case 'Зачет':
                $data->type = Model_Discipline::CREDIT;
                break;
            default:
                Log::instance()->add(Log::WARNING, "A discipline has incorrect type: {0}",
                    array(
                        '{0}'   => print_r($data, TRUE)
                    )
                );
                return null;
        }

        return $data;
    }

    private function normalizeStudyPlanData($data) {
        if ( !isset($data->id) && isset($data->externalID) ) {
            $ID = Model_Plan::withExternalID($data->externalID);
            if ( $ID != -1 )
                $data->id = $ID;
        }

        foreach ($data->disciplines as &$discipline) {
            $discipline = $this->normalizeDisciplinesData($discipline);
        }

        return $data;
    }

    private function createStudyPlan($data) {
        try {

        } catch (InvalidArgumentException $e) {
            $this->badRequestError( $this->makeErrorMsg($data, $e->getMessage()) );
        }
        return [];
    }

    private function updateStudyPlan($data) {
       $res = [];
        try {
            foreach ($data->disciplines as $discipline) {
                if ( $discipline == null )
                    continue;

                if ( isset($discipline->id) ) {
                    // ToDo: update discipline
                } else {
                    print_r($discipline);

                    $plan = Model_Plan::load($data->id);
                    $facultyID = $plan->FacultyID;
                    $semesterID = $plan->SemesterID;
                    $gradeID = $plan->GradeID;


                    $newSubjectID = Model_Subject::create($discipline->name, '', $facultyID);

                    $newDiscipline = Model_Discipline::make();
                    $newDiscipline->author($discipline->teacherIDs[0]);
                    $newDiscipline->subject($newSubjectID);
                    $newDiscipline->type($discipline->type);
                    $newDiscipline->semester($semesterID);
                    $newDiscipline->faculty($facultyID);
                    $newDiscipline->grade($gradeID);
                    $newDiscipline = $newDiscipline->create();
                    $res[] = $newDiscipline->ID;
                }
            }
        } catch (InvalidArgumentException $e) {
            $this->badRequestError( $this->makeErrorMsg($data, $e->getMessage()) );
        }
        return $res;
    }

    private function processData($data) {
        $data = $this->normalizeStudyPlanData($data);
        if ($data->id != -1) {
            return $this->updateStudyPlan($data);
        } else {
            return $this->createStudyPlan($data);
        }
    }

    /**
     * @api {put} api/v0/studyPlan Add new study plan(s)
     * @apiName Add new study plan
     * @apiGroup Study plans
     * @apiVersion 0.1.2
     * @apiParam {String} token Api key
     * @apiParam {String} [batch] Flag about massive addition
     * @apiParam {String} externalId
     * @apiParam {Array} disciplines
     */
    public function action_put_index() {
        try {
            if ( $this->request->query('batch') !== NULL ) {
                $data = json_decode($this->request->body());

                $res = [];
                foreach ($data as $item) {
                    $res[] = $this->processData($item);
                }
            } else {
                $data = $this->request->query();
                $res[] = $this->processData($data);
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }
}
